var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
import { h } from '../../../../base/browser/dom.js';
import { Disposable } from '../../../../base/common/lifecycle.js';
import { globalTransaction } from '../../../../base/common/observableInternal/base.js';
import { DiffEditorWidget } from '../diffEditor/diffEditorWidget.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { ILabelService } from '../../../../platform/label/common/label.js';
export class TemplateData {
    constructor(height, viewModel, entry) {
        this.height = height;
        this.viewModel = viewModel;
        this.entry = entry;
    }
    getId() {
        return this.entry;
    }
}
let DiffEditorItemTemplate = class DiffEditorItemTemplate extends Disposable {
    constructor(_container, _overflowWidgetsDomNode, _instantiationService, _labelService) {
        super();
        this._container = _container;
        this._overflowWidgetsDomNode = _overflowWidgetsDomNode;
        this._instantiationService = _instantiationService;
        this._labelService = _labelService;
        this._height = 500;
        this._heightObs = undefined;
        this._elements = h('div', {
            style: {
                display: 'flex',
                flexDirection: 'column',
            }
        }, [
            h('div', {
                style: {
                    display: 'flex',
                    flexDirection: 'column',
                    flex: '1',
                    border: '1px solid #4d4d4d',
                    borderRadius: '5px',
                    overflow: 'hidden',
                    margin: '10px 10px 10px 10px',
                }
            }, [
                h('div', { style: { display: 'flex', alignItems: 'center', padding: '8px 5px', background: 'var(--vscode-multiDiffEditor-headerBackground)', color: 'black' } }, [
                    //h('div.expand-button@collapseButton', { style: { margin: '0 5px' } }),
                    h('div@title', { style: { fontSize: '14px' } }, ['Title']),
                ]),
                h('div', {
                    style: {
                        flex: '1',
                        display: 'flex',
                        flexDirection: 'column',
                    }
                }, [
                    h('div@editor', { style: { flex: '1' } }),
                ])
            ])
        ]);
        this._editor = this._register(this._instantiationService.createInstance(DiffEditorWidget, this._elements.editor, {
            scrollBeyondLastLine: false,
            hideUnchangedRegions: {
                enabled: true,
            },
            scrollbar: {
                vertical: 'hidden',
                horizontal: 'visible',
                handleMouseWheel: false,
            },
            renderOverviewRuler: false,
            fixedOverflowWidgets: true,
            overflowWidgetsDomNode: this._overflowWidgetsDomNode, // TODO
        }, {}));
        // TODO@hediet
        /*
        const btn = new Button(this._elements.collapseButton, {});
        btn.icon = Codicon.chevronDown;
        */
        this._register(this._editor.onDidContentSizeChange(e => {
            this._height = e.contentHeight + this._elements.root.clientHeight - this._elements.editor.clientHeight;
            globalTransaction(tx => {
                var _a;
                (_a = this._heightObs) === null || _a === void 0 ? void 0 : _a.set(this._height, tx);
            });
        }));
        this._container.appendChild(this._elements.root);
    }
    setData(data) {
        this._heightObs = data.height;
        this._elements.title.innerText = this._labelService.getUriLabel(data.viewModel.model.modified.uri, { relative: true }); // data.entry.title;
        globalTransaction(tx => {
            this._editor.setModel(data.viewModel, tx);
            this._heightObs.set(this._height, tx);
        });
    }
    hide() {
        this._elements.root.style.top = `-100000px`;
        this._elements.root.style.visibility = 'hidden'; // Some editor parts are still visible
    }
    render(verticalRange, width, editorScroll) {
        this._elements.root.style.visibility = 'visible';
        this._elements.root.style.top = `${verticalRange.start}px`;
        this._elements.root.style.height = `${verticalRange.length}px`;
        this._elements.root.style.width = `${width}px`;
        this._elements.root.style.position = 'absolute';
        this._editor.layout({ width, height: verticalRange.length });
        this._editor.getOriginalEditor().setScrollTop(editorScroll);
    }
};
DiffEditorItemTemplate = __decorate([
    __param(2, IInstantiationService),
    __param(3, ILabelService)
], DiffEditorItemTemplate);
export { DiffEditorItemTemplate };
