/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
var __decorate = (this && this.__decorate) || function (decorators, target, key, desc) {
    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
    if (typeof Reflect === "object" && typeof Reflect.decorate === "function") r = Reflect.decorate(decorators, target, key, desc);
    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
    return c > 3 && r && Object.defineProperty(target, key, r), r;
};
var __param = (this && this.__param) || function (paramIndex, decorator) {
    return function (target, key) { decorator(target, key, paramIndex); }
};
import { getWindow, h, scheduleAtNextAnimationFrame } from '../../../../base/browser/dom.js';
import { SmoothScrollableElement } from '../../../../base/browser/ui/scrollbar/scrollableElement.js';
import { Disposable, toDisposable } from '../../../../base/common/lifecycle.js';
import { autorun, constObservable, derivedWithStore, observableFromEvent, observableValue } from '../../../../base/common/observable.js';
import { Scrollable } from '../../../../base/common/scrollable.js';
import './style.css';
import { DiffEditorWidget } from '../diffEditor/diffEditorWidget.js';
import { ObservableElementSizeObserver } from '../diffEditor/utils.js';
import { OffsetRange } from '../../../common/core/offsetRange.js';
import { IInstantiationService } from '../../../../platform/instantiation/common/instantiation.js';
import { DiffEditorItemTemplate, TemplateData } from './diffEditorItemTemplate.js';
import { ObjectPool } from './objectPool.js';
let MultiDiffEditorWidgetImpl = class MultiDiffEditorWidgetImpl extends Disposable {
    constructor(_element, _dimension, _model, _instantiationService) {
        super();
        this._element = _element;
        this._dimension = _dimension;
        this._model = _model;
        this._instantiationService = _instantiationService;
        this._elements = h('div', {
            style: {
                overflowY: 'hidden',
            }
        }, [
            h('div@content', {
                style: {
                    overflow: 'hidden',
                }
            }),
            h('div.monaco-editor@overflowWidgetsDomNode', {}),
        ]);
        this._sizeObserver = new ObservableElementSizeObserver(this._element, undefined);
        this._documentsObs = this._model.map(this, m => !m ? constObservable([]) : observableFromEvent(m.onDidChange, /** @description Documents changed */ () => m.diffs));
        this._documents = this._documentsObs.map(this, (m, reader) => m.read(reader));
        this._objectPool = new ObjectPool((data) => {
            const template = this._instantiationService.createInstance(DiffEditorItemTemplate, this._elements.content, this._elements.overflowWidgetsDomNode);
            template.setData(data);
            return template;
        });
        this._hiddenContainer = document.createElement('div');
        this._editor = this._register(this._instantiationService.createInstance(DiffEditorWidget, this._hiddenContainer, {
            hideUnchangedRegions: {
                enabled: true,
            },
        }, {}));
        this._viewItems = derivedWithStore(this, (reader, store) => this._documents.read(reader).map(d => store.add(new DiffEditorItem(this._objectPool, d, this._editor))));
        this._totalHeight = this._viewItems.map(this, (items, reader) => items.reduce((r, i) => r + i.contentHeight.read(reader), 0));
        this._sizeObserver.setAutomaticLayout(true);
        this._register(autorun((reader) => {
            /** @description Update widget dimension */
            const dimension = this._dimension.read(reader);
            this._sizeObserver.observe(dimension);
        }));
        const scrollable = this._register(new Scrollable({
            forceIntegerValues: false,
            scheduleAtNextAnimationFrame: (cb) => scheduleAtNextAnimationFrame(getWindow(_element), cb),
            smoothScrollDuration: 100,
        }));
        this._elements.content.style.position = 'relative';
        const scrollableElement = this._register(new SmoothScrollableElement(this._elements.root, {
            vertical: 1 /* ScrollbarVisibility.Auto */,
            className: 'monaco-component',
        }, scrollable));
        this.scrollTop = observableFromEvent(scrollableElement.onScroll, () => /** @description onScroll */ scrollableElement.getScrollPosition().scrollTop);
        this._register(autorun((reader) => {
            /** @description Update scroll dimensions */
            const height = this._sizeObserver.height.read(reader);
            this._elements.root.style.height = `${height}px`;
            const totalHeight = this._totalHeight.read(reader);
            this._elements.content.style.height = `${totalHeight}px`;
            scrollableElement.setScrollDimensions({
                width: _element.clientWidth,
                height: height,
                scrollHeight: totalHeight,
                scrollWidth: _element.clientWidth,
            });
        }));
        _element.replaceChildren(scrollableElement.getDomNode());
        this._register(toDisposable(() => {
            _element.replaceChildren();
        }));
        this._register(this._register(autorun(reader => {
            /** @description Render all */
            this.render(reader);
        })));
    }
    render(reader) {
        const scrollTop = this.scrollTop.read(reader);
        let contentScrollOffsetToScrollOffset = 0;
        let itemHeightSumBefore = 0;
        let itemContentHeightSumBefore = 0;
        const viewPortHeight = this._elements.root.clientHeight;
        const contentViewPort = OffsetRange.ofStartAndLength(scrollTop, viewPortHeight);
        const width = this._elements.content.clientWidth;
        for (const v of this._viewItems.read(reader)) {
            const itemContentHeight = v.contentHeight.read(reader);
            const itemHeight = Math.min(itemContentHeight, viewPortHeight);
            const itemRange = OffsetRange.ofStartAndLength(itemHeightSumBefore, itemHeight);
            const itemContentRange = OffsetRange.ofStartAndLength(itemContentHeightSumBefore, itemContentHeight);
            if (itemContentRange.isBefore(contentViewPort)) {
                contentScrollOffsetToScrollOffset -= itemContentHeight - itemHeight;
                v.hide();
            }
            else if (itemContentRange.isAfter(contentViewPort)) {
                v.hide();
            }
            else {
                const scroll = Math.max(0, Math.min(contentViewPort.start - itemContentRange.start, itemContentHeight - itemHeight));
                v.render(itemRange, scroll, width);
                contentScrollOffsetToScrollOffset -= scroll;
            }
            itemHeightSumBefore += itemHeight;
            itemContentHeightSumBefore += itemContentHeight;
        }
        this._elements.content.style.transform = `translateY(${-(scrollTop + contentScrollOffsetToScrollOffset)}px)`;
    }
};
MultiDiffEditorWidgetImpl = __decorate([
    __param(3, IInstantiationService)
], MultiDiffEditorWidgetImpl);
export { MultiDiffEditorWidgetImpl };
class DiffEditorItem extends Disposable {
    constructor(_objectPool, _entry, baseDiffEditorWidget) {
        super();
        this._objectPool = _objectPool;
        this._entry = _entry;
        this._height = observableValue(this, 500);
        this.contentHeight = this._height;
        this._vm = this._register(baseDiffEditorWidget.createViewModel({
            original: _entry.value.original,
            modified: _entry.value.modified,
        }));
    }
    dispose() {
        this.hide();
        super.dispose();
    }
    toString() {
        return `ViewItem(${this._entry.value.title})`;
    }
    hide() {
        var _a, _b;
        (_a = this._templateRef) === null || _a === void 0 ? void 0 : _a.object.hide();
        (_b = this._templateRef) === null || _b === void 0 ? void 0 : _b.dispose();
        this._templateRef = undefined;
    }
    render(verticalSpace, offset, width) {
        if (!this._templateRef) {
            this._templateRef = this._objectPool.getUnusedObj(new TemplateData(this._height, this._vm, this._entry.value));
        }
        this._templateRef.object.render(verticalSpace, width, offset);
    }
}
